import random

import pygame
from pygame.locals import *

import imagecache
import imagecachedb


def make_images():
    if IMAGE_DB.count():
        print('make_images: database has images: skipping creation')
        return
    print('make_images: making images')
    # create a base image
    base_image = pygame.Surface(IMAGE_SIZE)
    base_image.fill(Color(IMAGE_NAME))
    rect = base_image.get_rect()
    pygame.draw.rect(base_image, Color('midnightblue'), rect, 1)
    black = Color('black')
    # rotate the image and apply a colorkey
    params = []
    for angle in range(360):
        image = pygame.transform.rotozoom(base_image, angle, 1.0)
        image.set_colorkey(black)
        params.append((IMAGE_NAME, angle, image))
    # store images in the database
    print('make_images: saving {} images in database'.format(len(params)))
    IMAGE_DB.insert_many(params)


def load_images():
    print('load_images: loading images from database')
    IMAGE_DB.load(IMAGE_CACHE)


def make_sprites(num):
    new_sprites = []
    x1, x2 = IMAGE_SIZE[0] // 2, RESOLUTION[0] - IMAGE_SIZE[0] // 2
    y1, y2 = IMAGE_SIZE[1] // 2, RESOLUTION[1] - IMAGE_SIZE[1] // 2
    for i in range(num):
        rect = pygame.Rect((0, 0), IMAGE_SIZE)
        rect.center = random.randint(x1, x2), random.randint(y1, y2)
        angle = random.randint(0, 359)
        new_sprites.append(Sprite(rect, angle))
    return new_sprites


def draw_sprites(surf, sprites):
    get = IMAGE_CACHE.get
    name = IMAGE_NAME
    blit = surf.blit
    for sprite in sprites:
        sprite.angle = (sprite.angle + sprite.speed) % 360

        im = get(name, int(sprite.angle))

        rect = sprite.rect
        center = rect.center
        rect.size = im.get_size()
        rect.center = center

        blit(im, rect)


class Sprite(object):

    __slots__ = 'rect', 'angle', 'speed'

    def __init__(self, rect, angle):
        self.rect = rect
        self.angle = angle
        self.speed = 0.1 + 1.9 * random.random()


pygame.init()

megabyte = 2 ** 20

IMAGE_CACHE = imagecache.ImageCache(default_expiration=5.0, max_memory=100 * megabyte)
IMAGE_DB = imagecachedb.ImageCacheDB('example2-db.sqlite')
IMAGE_CACHE.enable_memory_cap(False)

IMAGE_NAME = 'orange'
IMAGE_SIZE = 64, 64
NUM_SPRITES = 25
RESOLUTION = 1024, 768

screen = pygame.display.set_mode(RESOLUTION)

clock = pygame.time.Clock()
max_fps = 0
countdown = 0.25

sprites = make_sprites(NUM_SPRITES)

make_images()
load_images()

running = True
while running:
    screen.fill((0, 0, 0))
    draw_sprites(screen, sprites)
    pygame.display.flip()

    for e in pygame.event.get():
        if e.type == KEYDOWN:
            if e.key == K_ESCAPE:
                running = False
        elif e.type == QUIT:
            running = False

    dt = clock.tick(max_fps) / 1000.0
    countdown -= dt
    if countdown <= 0.0:
        countdown += 0.5
        pygame.display.set_caption('{} sprites | {} fps | {} entries | {} MB | {}% hit'.format(
            len(sprites),
            int(clock.get_fps()),
            IMAGE_CACHE.count(),
            int(IMAGE_CACHE.size() / megabyte),
            int(IMAGE_CACHE.hit_ratio())))
        if clock.get_fps() > 60:
            sprites.extend(make_sprites(1))

    IMAGE_CACHE.update()
