import pygame
from pygame.locals import *

import sqlnub


def init_db():
    global db
    db = sqlnub.SqlNub('images.sqlite')
    if 'Images' not in db.get_table_names():
        print('creating data')
        db.create_table(
            'Images', (
                ('name', 'text'),
                ('transforms', 'text'),
                ('width', 'int'),
                ('height', 'int'),
                ('data', 'text'),
                ('primary key', "('name', 'transforms')"),
            ))
        db.execute('create index image_index on Images(name, transforms)')
        orig = pygame.surface.Surface((64, 64))
        orig.fill(Color('orange'))
        for angle in range(0, 360):
            im = pygame.transform.rotozoom(orig, angle, 1.0)
            width, height = im.get_size()
            insert_image('orange', angle, width, height, im)
        db.commit()
    return db


def insert_image(name, angle, width, height, im):
    data = pygame.image.tostring(im, 'RGBA')
    db.execute("""
        insert into Images
        (name, transforms, width, height, data)
        values (?, ?, ?, ?, ?)
        """, (name, angle, width, height, buffer(data)))


def query_image(angle, rect):
    cursor = db.execute("""
        select * from Images
        indexed by image_index
        where name=? and transforms=?
        """, ('orange', angle))
    row = cursor.fetchone()
    if not row:
        return None, None
    data = row['data']
    size = row['width'], row['height']
    im = pygame.image.fromstring(str(data), size, 'RGBA')

    center = rect.center
    rect.size = size
    rect.center = center
    return im, rect


pygame.init()

resolution = 400, 400
screen = pygame.display.set_mode(resolution)
screen_rect = screen.get_rect()

clock = pygame.time.Clock()
max_fps = 0
countdown = 2.0

rect = pygame.Rect(0, 0, 64, 64)
rect.center = 200, 200
angle = 0

db = init_db()

running = True
while running:
    angle = (angle + 1) % 360
    im, rect = query_image(angle, rect)

    screen.fill((0, 0, 0))
    screen.blit(im, rect)
    pygame.display.flip()

    for e in pygame.event.get():
        if e.type == KEYDOWN:
            if e.key == K_ESCAPE:
                running = False
        elif e.type == QUIT:
            running = False

    dt = clock.tick(max_fps) / 1000.0
    countdown -= dt
    if countdown <= 0.0:
        countdown += 2.0
        pygame.display.set_caption('{} fps'.format(int(clock.get_fps())))
