import os
import sys
sys.path.insert(0, '..')

import pygame
from pygame.locals import *

import imagecachedb


pygame.init()

RESOLUTION = 400, 400
DB_NAME = (':memory:', 'test.sqlite')[0]
DB_TABLES = (None, 'table1')
IMAGE_NAME = 'green'
TRANSFORMS = (
    90,
    95.0,
    (100,),
)

screen = pygame.display.set_mode(RESOLUTION)
screen_rect = screen.get_rect()

image = pygame.Surface((64, 64))
image.fill(Color(IMAGE_NAME))
rect = image.get_rect()

if os.access(DB_NAME, os.F_OK):
    os.unlink(DB_NAME)


def display(message, image):
    pygame.display.set_caption(message)
    rect = image.get_rect(center=screen_rect.center)
    screen.fill((0, 0, 0))
    screen.blit(image, rect)
    pygame.display.flip()


def test_one():
    print('\n## TEST ONE')
    db = imagecachedb.ImageCacheDB(DB_NAME)

    for table_name in DB_TABLES:
        print('\nTable: {}'.format(table_name))

        if table_name is not None:
            db.create_images_table(table_name)

        for transform in TRANSFORMS:
            try:
                t = transform[0]
            except TypeError:
                t = transform

            message = 'INSERT {} {} {}'.format(table_name, IMAGE_NAME, transform)
            print(message)
            im = pygame.transform.rotozoom(image, t, 1.0)
            display(message, im)
            db.insert(IMAGE_NAME, transform, im, table_name)
            pygame.time.wait(1000)

        assert db.count(table_name) == 3

        for transform in TRANSFORMS:
            message = 'GET {} {} {}'.format(table_name, IMAGE_NAME, transform)
            print(message)
            im = db.get(IMAGE_NAME, transform, table_name)
            display(message, im)

            message = 'DELETE {} {} {}'.format(table_name, IMAGE_NAME, transform)
            print(message)
            db.delete(IMAGE_NAME, transform, table_name)
            pygame.time.wait(1000)

        assert db.count(table_name) == 0


def test_many():
    print('\n## TEST MANY\n')
    db = imagecachedb.ImageCacheDB(DB_NAME)

    info2 = {}  # {table1: [(name1, transforms1), ...], ...}
    info3 = {}  # {table1: [(name1, transforms1, image1), ...], ...}

    for table_name in DB_TABLES:
        print('Table insert many: {}'.format(table_name))

        if table_name is not None:
            db.create_images_table(table_name)

        info2[table_name] = []
        info3[table_name] = []
        for transform in TRANSFORMS:
            try:
                t = transform[0]
            except TypeError:
                t = transform
            im = pygame.transform.rotozoom(image, t, 1.0)
            info2[table_name].append((IMAGE_NAME, transform))
            info3[table_name].append((IMAGE_NAME, transform, im))
        db.insert_many(info3[table_name], table_name)
        assert db.count(table_name) == 3
        print('NAMES {}: {}'.format(table_name, db.get_names(table_name)))

    for table_name in DB_TABLES:
        print('\nTable get many: {}'.format(table_name))
        print('Info2 {}: {}'.format(table_name, info2[table_name]))
        images = db.get_many(info2[table_name], table_name)
        for im, (n, t) in zip(images, info2[table_name]):
            message = 'GET MANY {}: {} {}'.format(table_name, n, t)
            print(message)
            display(message, im)
            pygame.time.wait(1000)

    for table_name in DB_TABLES:
        print('\nTable get by name: {}'.format(table_name))
        print('Info2 {}: {}'.format(table_name, info2[table_name]))
        images = db.get_by_name(IMAGE_NAME, table_name)
        for im, (n, t) in zip(images, info2[table_name]):
            message = 'GET BY NAME {}: {} {}'.format(table_name, n, t)
            print(message)
            display(message, im)
            pygame.time.wait(1000)

    for table_name in DB_TABLES:
        print('\nTable delete many: {}'.format(table_name))
        print('Info2 {}: {}'.format(table_name, info2[table_name]))
        if table_name is None:
            db.delete_many(info2[table_name], table_name)
            fmt = 'DELETE MANY {}: {} {}'
        else:
            db.delete_by_name(IMAGE_NAME, table_name)
            fmt = 'DELETE BY NAME {}: {} {}'
        for n, t in info2[table_name]:
            message = fmt.format(table_name, n, t)
            print(message)
            assert db.count(table_name) == 0


test_one()
test_many()
